<?php
/**
 * Plugin Messages (addon for plugin Adm_redacteurs)
 * Version  : 1.1 (2017/10/23)
 * Compatibility : Guppy v5.0+ et Adm_redacteurs v6.8+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

require_once ADMR_HELPERS_PATH.'/search_records_helper.php';

class Search_messages_helper extends Search_records_helper
{
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    protected $CI; // object global
    protected $_lang = array();          // Données de langue
    protected $_config = array();        // Données de configuration
    protected $_params = array();        // Données récupérés par post ou get
    protected $_writer_rights = array(); // Droits du rédacteur    
    protected $_doc_infos = array('name' => 'messages', 'type' => 'messages'); // Tableau contenant les informations du document (nom, type)

    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */

	/**
	 * constructeur
     * 
     * @access public
     * @param array Tableau contenant les droits du rédacteur
	 * @return void
	 */	
    public function __construct($writer_rights)
    {
        parent :: __construct($this -> _doc_infos, $writer_rights);
        
		$this -> CI -> load_model('writer_model');
        $this -> CI -> load_addon_model('messages', 'writer_messages_model', array('id' => $this -> _writer_rights['id'])); // Model des documents de guppy
        $this -> CI -> load_addon_helper('messages', 'plugin_menu_for_messages_helper');
        $this -> CI -> load_addon_helper('messages', 'messages_infos_helper');
        
        $this -> CI -> load_addon_lang('messages');
        $this -> CI -> load_addon_config('messages');        

        $this -> _lang = $this -> CI -> get_lang();
        $this -> _config = $this -> CI -> get_config();
    }

    /*
    |--------------------------------------------------------------------------
    | Methods
    |--------------------------------------------------------------------------
    */

	/**
	 * recherche des enregistrements et redirige vers la page qui va afficher les résultats de la recherche
     * 
     * @access public
	 * @return void
	 */
    public function search()
    {
        return parent :: search();   
    }

	// --------------------------------------------------------------------

    /**
     * Définit et renvoie le titre de la page 
     *
     * @param array Données à faire passer à la vue
     * @return void
     */        
    public function set_title_page()
    {
        return ucfirst($this -> _lang['search_messages']); 
    }
    
	// --------------------------------------------------------------------

	/**
	 * définit et renvoie la description de la page ainsi que les recommendations
     * 
     * @access public
	 * @return string description de la page
	 */	
    public function set_infos_page_content()
    {		
        return '';
    }

	// --------------------------------------------------------------------

	/**
	 * définit et renvoie les ajouts (js et css) dans l'entête de la page
     * 
     * @access public
	 * @return string Chaine html contenant les ajouts js et css
	 */	
    public function set_header_addons()
    {        
        $js = '';
        //$js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/'.$prefixe.'manage_records.js"></script>'."\n";

        $css = '';
        //$css = '<link type="text/css" rel="stylesheet" href="'.ADMR_CSS_PATH.'/'.$prefixe.'manage_records.css" />';

        return $css.$js;        
    }

	// --------------------------------------------------------------------

    /*
     * Définit le contenu du menu qui est affiché dans le bandeau en haut de la page
     *
     * @return array Tableau contenant les items du menu (vide par défaut)
     */
    public function set_menu_plugin_items()
    {        
        if (ADMR_IS_ADMIN) {
            return Plugin_menu_for_messages_helper :: set_admin_items('messages', $this -> _lang, $this -> _config, $this -> _writer_rights);
        } else {
            return Plugin_menu_for_messages_helper :: set_public_items('messages', $this -> _lang, $this -> _config, $this -> _writer_rights);
        }
    }

	// --------------------------------------------------------------------

    /**
     * Définit le contenu des informations du plugin qui sont affichées en bas de la page
     *
     * @return array Tableau contenant les informations du plugin sous la forme array('version' => , 'licence' => )
     */
    public function set_infos_plugin()
    {
        return Messages_infos_helper :: set_plugin_infos($this -> _lang);
    }

	// --------------------------------------------------------------------

    /**
	 * Définit et retourne des données supplémentaires à faire passer à la vue
     * 
	 * @access protected
     * @return array Tableau de données à faire passer à la vue
	 */	
    protected function _get_more_data_for_search_form()
    {
       return array(
            'writer_messages_sent' => $this -> _get_writer_messages_sent(),
            'writer_messages_received' => $this -> _get_writer_messages_received(),      
            'writers_for_messages_received' => $this -> _get_writers_for_messages_received(),
            'writers_for_messages_sent' => $this -> _get_receivers_messages_sent(),
       ); 
    }   

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des auteurs des messages reçus au rédacteur
	 *
	 * @access	protected
	 * @return	array sous la forme array(id => name)
	 */
    protected function _get_writers_for_messages_received()
    {        
        return $this -> _get_writers_for_messages('received');    
    }

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des destinataires des messages envoyés au rédacteur
	 *
	 * @access	protected
	 * @return	array sous la forme array(id => name)
	 */
    protected function _get_receivers_messages_sent()
    {        
        return $this -> _get_writers_for_messages('sent');    
    }

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des auteurs des messages envoyés au rédacteur
	 *
	 * @access	protected
	 * @return	array sous la forme array(id => name)
	 */
    protected function _get_writers_for_messages($messages_type = 'received')
    {
        $writers = array();
        $writer_ids = array();
        $messages = array();
        $this -> CI -> load_model('writer_model');
        $this -> CI -> load_helper('array_helper');
        
        if ($messages_type == 'received') {
            $messages = $this -> _get_writer_messages_received();
            $writer_ids = array_column($messages, 'author_id');
        } else {
            $messages = $this -> _get_writer_messages_sent();
            $writer_ids = array_column($messages, 'receiver_id');  
        }
       
        $writers = $this -> CI -> writer_model -> get(array('id' => $writer_ids));
              
        return $writers;    
    }

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des messages envoyés du rédacteur
	 *
	 * @access	protected
	 * @return	array sous la forme array(id => name)
	 */
    protected function _get_writer_messages_sent()
    {
        $docs = array();
        
        $docs = $this -> CI -> writer_messages_model -> get(array('author_id' => array($this -> _writer_rights['id'])));
        
        foreach($docs as $key => $doc) {
            $docs[$key]['heading'] = $doc['title'];    
        }
        
        return $docs;    
    }

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des messages reçus du rédacteur
	 *
	 * @access	protected
	 * @return	array sous la forme array(id => name)
	 */
    protected function _get_writer_messages_received()
    {
        $docs = array();
        
        $docs = $this -> CI -> writer_messages_model -> get(array('receiver_id' => array($this -> _writer_rights['id'])));
        
        foreach($docs as $key => $doc) {
            $docs[$key]['heading'] = $doc['title'];    
        }
        
        return $docs;    
    }
    
	// --------------------------------------------------------------------

	/**
	 * définit les variables utilisées dans la page 
     * 
     * @access protected
	 * @return void
	 */	
    protected function _set_params()
    {
        return $this -> _params = array_merge(
            parent :: _set_params(),
            array(            
                'msg_received_ids' => Var_helper :: import('msg_received_ids', 'post', array()),
                'msg_sent_ids' => Var_helper :: import('msg_sent_ids', 'post', array()),
                'author_ids' => Var_helper :: import('author_ids', 'post', array()),
                'receiver_ids' => Var_helper :: import('receiver_ids', 'post', array()),
            )
        );
    }

	// --------------------------------------------------------------------

	/**
	 * Fait une redirection selon le résultat du filtre appliqué
	 *
	 * @access	protected
     * @param integer Identifiant du filtre appliqué
     * @param string Uri de redirection
	 * @return void
	 */
    protected function _redirect_by_filter($filter, $uri)
    {       
        $uri = 'addons/messages/'.$uri;
       
        /**
		 * Sélection des messages reçus
		 */    
        if ($filter % 2 == 0) { // Filtres pairs

    		/**
    		 * Sélection des messages reçus par leurs identifiants ($filter = 1)
    		 */    
            $messages_received_ids = $this -> _params['msg_received_ids'];
            
            if ($filter == '2' && !empty($messages_received_ids)) {
                $uri .= is_array($messages_received_ids) ? '&msg_received_ids='.implode('-', $messages_received_ids) : ''; 
            }

            /**
             * Sélection de messages reçus selon l'id des auteurs
             */
            $author_ids = $this -> _params['author_ids'];
                
            if ($filter == '4' && !empty($author_ids)) {
                $uri .= is_array($author_ids) ? '&author_ids='.implode('-', $author_ids) : ''; 
            }
            //
        }
        /**
		 * Sélection des messages envoyés
		 */         
        else { // Filtres impairs
            $uri .= '_sent';
            
    		/**
    		 * Sélection des messages envoyés par leurs identifiants ($filter = 1)
    		 */    
            $messages_sent_ids = $this -> _params['msg_sent_ids'];
            
            if ($filter == '3' && !empty($messages_sent_ids)) {
                $uri .= is_array($messages_sent_ids) ? '&msg_sent_ids='.implode('-', $messages_sent_ids) : ''; 
            }
            
            /**
             * Sélection de messages envoyés selon l'id des destinataires
             */
            $receiver_ids = $this -> _params['receiver_ids'];
                
            if ($filter == '5' && !empty($receiver_ids)) {
                $uri .= is_array($receiver_ids) ? '&receiver_ids='.implode('-', $receiver_ids) : ''; 
            }
            //
        }
        
        // Redirection        
        Url_helper :: redirect($uri);
    }

	// --------------------------------------------------------------------

    /**
	 * Définit et retourne le layout de la vue
     * 
	 * @access protected
     * @return string nom du fichier qui définit le layout
	 */	
    protected function _get_layout_for_search_records_view()
    {
        return str_replace(ADMR_PUBLIC_ROOT_PATH, '..', MESSAGES_VIEWS_PATH).'/search_messages_layout';
    }

	// --------------------------------------------------------------------

} // End of Class